"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const moveNodeUtil_1 = require("./moveNodeUtil");
function moveNode(node, destFile, project) {
    // we copy all the imports from node.getSourceFile() to destFile and the organize imports so the moved declaration don't miss any import
    moveNodeUtil_1.fixImportsInDestFile(node, destFile);
    // For each sourceFile that reference node, we add an import declaration to node but specifying nodeFile. (we "move" the oriinal import declarations only changing the specifier to point to destFile).
    moveNodeUtil_1.fixImportsInReferencingFiles(node, destFile);
    // move the declaration - first add a copy of the node to destFile
    let finalNode;
    if (ts_simple_ast_1.TypeGuards.isClassDeclaration(node)) {
        finalNode = destFile.addClass(node.getStructure());
    }
    else if (ts_simple_ast_1.TypeGuards.isInterfaceDeclaration(node)) {
        finalNode = destFile.addInterface(node.getStructure());
    }
    else if (ts_simple_ast_1.TypeGuards.isFunctionDeclaration(node)) {
        finalNode = destFile.addFunction(node.getStructure());
    }
    const nodeIsDefaultExport = node.isDefaultExport();
    // and then remove node from its sourceFile
    node.remove();
    finalNode.setIsExported(true);
    finalNode.setIsDefaultExport(nodeIsDefaultExport);
    moveNodeUtil_1.safeOrganizeImports(destFile);
    moveNodeUtil_1.safeOrganizeImports(node.getSourceFile());
}
exports.moveNode = moveNode;
//# sourceMappingURL=moveNode.js.map