"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const assert_1 = require("assert");
const moveNode_1 = require("./moveNode");
function moveNodeTest() {
    const project = new ts_simple_ast_1.Project({ useVirtualFileSystem: true });
    const lionFile = project.createSourceFile('src/animal/lion/Lion.ts', `
  import {Food} from '../../food/Food'
  import {Animal} from '../Animal'
  export class Lion extends Animal{
    eat(meat: Food){}
  }
  `);
    const animalFile = project.createSourceFile('src/animal/Animal.ts', `
  export class Animal {
    breath(air: number){}
  }
  `);
    const energyFile = project.createSourceFile('src/energy/Energy.ts', `
  export class Energy {
    burn(): number {return 1}
  }
  `);
    const foodFile = project.createSourceFile('src/food/Food.ts', `
  import {Animal} from '../animal/Animal'
  import {Energy} from '../energy/Energy'
  export class Food {
    energy: Energy
    canEatBy: Animal[]
  }
  `);
    const Food = foodFile.getClass('Food');
    const tmpFile = project.createSourceFile('tmp.ts', '');
    // console.log('ERRORS: '+project.getPreEmitDiagnostics().map(e=>e.getMessageText()+' - '+(e.getSourceFile() && e.getSourceFile().getFilePath())).join('\n'))
    moveNode_1.moveNode(Food, animalFile);
    // console.log(printSourceFile(lionFile));
    assert_1.equal(printSourceFile(foodFile), '');
    assert_1.equal(printSourceFile(animalFile), 'import { Energy } from "../energy/Energy"; export class Animal { breath(air: number){} } export class Food { energy: Energy; canEatBy: Animal[]; }');
    assert_1.equal(printSourceFile(lionFile), `import {Animal} from '../Animal' import { Food } from "../animal/Animal"; export class Lion extends Animal{ eat(meat: Food){} }`);
}
exports.moveNodeTest = moveNodeTest;
function printSourceFile(sf) {
    return sf.getText().trim().replace(/[\s]+/gm, ' ');
}
//# sourceMappingURL=moveNodeTest.js.map