"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const typescript_ast_util_1 = require("typescript-ast-util");
const testUtil_1 = require("../spec/testUtil");
function fixImportsInDestFile(node, destFile) {
    const importsToAddToDestFile = node.getSourceFile().getImportDeclarations().filter(i => i.getModuleSpecifierSourceFile() !== destFile).map(i => {
        const specifiedFile = i.getModuleSpecifierSourceFile();
        const isLibrary = !specifiedFile || !i.getModuleSpecifierValue().trim().startsWith('.');
        const moduleSpecifier = isLibrary ? i.getModuleSpecifierValue() : destFile.getRelativePathAsModuleSpecifierTo(specifiedFile);
        return Object.assign({}, i.getStructure(), { moduleSpecifier });
    });
    // import all nodes used by node that are declared outside it but not imported . Make sure they are exported
    const referencedByNodeNotImported = findReferencesDeclaredOutside(node)
        .filter(r => r.getSourceFile() === node.getSourceFile() && !r.getFirstAncestor(ts_simple_ast_1.TypeGuards.isImportDeclaration))
        .map(n => {
        const exportableAncestor = n.getFirstAncestor(a => ts_simple_ast_1.TypeGuards.isExportableNode(a) && ts_simple_ast_1.TypeGuards.isNameableNode(a) && a.getName() === n.getText());
        if (exportableAncestor) {
            exportableAncestor.setIsExported(true);
        }
        return n;
    })
        .filter((n, i, a) => a.indexOf(n) === i);
    referencedByNodeNotImported.forEach(n => {
        importsToAddToDestFile.push({
            moduleSpecifier: destFile.getRelativePathAsModuleSpecifierTo(node.getSourceFile()),
            namedImports: [{ name: n.getText() }]
        });
    });
    destFile.addImportDeclarations(importsToAddToDestFile);
}
exports.fixImportsInDestFile = fixImportsInDestFile;
function fixImportsInReferencingFiles(node, destFile) {
    // first obtain all the files referencing (importing) node 
    const referencedSourceFiles = getReferences(node)
        .map(r => r.getSourceFile())
        .filter((f, i, a) => a.indexOf(f) === i);
    referencedSourceFiles.forEach(f => {
        const newImports = f.getImportDeclarations()
            .filter(i => i.getModuleSpecifierSourceFile() === node.getSourceFile())
            .map(i => {
            const s = i.getStructure();
            // get the named input but only for node (in case there are multiple names in the import we remove the others)
            let namedImports;
            if (s.namedImports && Array.isArray(s.namedImports)) {
                namedImports = s.namedImports.filter(ni => typeof (ni) === 'string' ? ni === node.getName() : ni.name === node.getName());
            }
            else if (s.namedImports) {
                // console.log('ELSE');
                namedImports = [{ name: node.getName() }]; // and good luck!
                //TODO: what is the case when namedImports is of type writerFunction, not an array ? how do I handle that ? - an structure shound be serializable and here a field can be a function !!! 
            }
            return Object.assign({}, s, { moduleSpecifier: f.getRelativePathAsModuleSpecifierTo(destFile), namedImports });
        });
        f.addImportDeclarations(newImports);
        f.getImportDeclarations().filter(i => i.getModuleSpecifierSourceFile() === node.getSourceFile()).forEach(i => i.remove());
    });
    node.getSourceFile().addImportDeclaration({
        moduleSpecifier: node.getSourceFile().getRelativePathAsModuleSpecifierTo(destFile),
        namedImports: node.isDefaultExport() ? undefined : [{ name: node.getName() }],
        defaultImport: node.isDefaultExport() ? node.getName() : undefined,
    });
}
exports.fixImportsInReferencingFiles = fixImportsInReferencingFiles;
let tmpSourceFile;
function safeOrganizeImports(f) {
    if (!tmpSourceFile) {
        const project = testUtil_1.createProject();
        tmpSourceFile = project.createSourceFile(`tmp_${new Date().getTime()}.ts`, '');
    }
    tmpSourceFile.replaceWithText(f.getText());
    tmpSourceFile.organizeImports();
    f.replaceWithText(tmpSourceFile.getText());
}
exports.safeOrganizeImports = safeOrganizeImports;
function getReferences(node) {
    const references = [];
    const referencedSymbols = node.findReferences();
    for (const referencedSymbol of referencedSymbols) {
        for (const reference of referencedSymbol.getReferences()) {
            references.push(reference);
        }
    }
    return references;
}
exports.getReferences = getReferences;
function findReferencesDeclaredOutside(node, outside = true) {
    const refs = node.getDescendants().filter(ts_simple_ast_1.TypeGuards.isReferenceFindableNode)
        .map(d => {
        return getReferences(d)
            .filter(r => r.isDefinition() && (outside ? !r.getNode().getFirstAncestor(a => a === node) : !!r.getNode().getFirstAncestor(a => a === node)))
            .map(r => r.getNode());
        // .filter(r => (outside ? !r.getNode().getFirstAncestor(a => a === node) : !!r.getNode().getFirstAncestor(a => a === node))).map(r => r.getNode())
    });
    return typescript_ast_util_1.flat(refs).filter((n, i, a) => a.indexOf(n) === i);
}
exports.findReferencesDeclaredOutside = findReferencesDeclaredOutside;
//# sourceMappingURL=moveNodeUtil.js.map